
#ifndef MIXX_H
#define MIXX_H

#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <mmsystem.h>

class Thread
{
public:
   Thread(void);               // Constructor initializes object. No new thread is created.
   ~Thread();                  // Destructor will terminate the thread if it is running and delete the thread object.
   BOOL Start();               // Start creates the new thread and returns TRUE or FALSE on success or failure respectively. The new thread starts running immediately.
   HANDLE GetThreadHandle();   // GetThreadHandle and GetThreadId return the thread handle and thread identification number respectively.
   DWORD GetThreadId();
   int GetPriority();          // GetPriority returns the current priority of the thread. Values range from 0 to 30.
   BOOL SetPriority(int p);    // SetPriority changes the priority of the thread. Values for p range from 0 to 30. Returns TRUE or FALSE to indicate success or not.
   BOOL Suspend();             // Suspend and Resume stop and restart execution of the thread. Return TRUE or FALSE to indicate success.
   BOOL Resume();
   BOOL Terminate(DWORD ec=0); // Terminate terminates the thread. Returns TRUE or FALSE to indicate success. ec is the return value of the thread.
   virtual void Run(void)=0;   // Run is the function the thread will execute when started. This is a placeholder for the user procedure. The thread is terminated when Run terminates.

private:
   static DWORD TProc(LPDWORD adr);
   HANDLE HThread;
   DWORD ThreadId;
};

#define SEM_INFINITE INFINITE

class Semaphore
{
public:
   Semaphore();                              // Constructor creates a new semaphore.
   ~Semaphore();                             // Destructor deletes the semaphore.
   BOOL Acquire(DWORD timeout=SEM_INFINITE); // Acquire waits until the semaphore is free and then allocates it for the current thread. timeout is the time to wait for the semaphore in milliseconds, or INFINITE. Returns TRUE or FALSE.
   BOOL Release();                           // Release frees the semaphore. Returns TRUE or FALSE.
   HANDLE GetHandle();                       // GetHandle returns the Semaphore handle.

private:
   HANDLE HSem;
};

class Buffer
{
public:
   Buffer(unsigned int long=0);                // Constructor allocates sz bytes of buffer space.
   ~Buffer();                                  // Destructor deallocates the buffer
   void SetBufferSize(unsigned long sz);
   unsigned long GetBufferSize(void) { return size; }
   char *QueryFree(unsigned long sz);           // QueryFree checks to see if sz bytes are available in the buffer. If so, it returns the start address of the block, if not NULL.
   char *QueryValid(unsigned long& sz);         // QueryValid returns the startadress and size (in sz) of data in the buffer. If no data is present, it returns NULL.
   void Brought(unsigned long n);               // Brought and Taken notify the buffer of data written into or read out of the buffer. n is the number of bytes.
   void Taken(unsigned long n);
   int GetFillLevel();                         // GetFillLevel returns a number between 0 and 100 indicating the procentual fill level of the buffer.
   virtual void NotifyBrought(unsigned long n); // Notification functions are called when data is brought or taken. n is the number of bytes brought or taken.
   virtual void NotifyTaken(unsigned long n);
   void Clear(void);

   char* GetData1(void) { return tail; }
   unsigned long GetSize1(void) { return tail?end-tail+1:0; }
   char* GetData2(void) { return head>tail?0:startadress; }
   unsigned long GetSize2(void) { return head>tail?0:head-startadress; }
   unsigned long GetSize(void) { return GetSize1()+GetSize2(); }

private:
   char *head,*tail,*end;
   char *startadress,*endadress;

   Semaphore sem;
   char *wrapsavehead;
   BOOL wrap;
   unsigned long size, datasize;
};

class Transport
{
public:
   Transport(int t=SOCK_DGRAM);           // Contructor initializes the Sockets library and allocates a datagram socket.
   ~Transport();                          // Destructor frees the socket and shuts down the sockets library
   BOOL Connect(char *host,u_short port); // Connect connects the socket to the specified host and port. If only a port is given, the socket can accept data from any host. The hostname can be a name or an adress in dotted decimal form. The returnvalue is TRUE if successful.
   BOOL Connect(u_short port); 
   void Close(void);
   void Open(void);
   int Send(char *buf,int siz);           // Send sends siz bytes from buf. Return number of bytes actually sent.
   int Receive(char *buf,int sz);         // Receive receives up to sz bytes in buf. Returns number of bytes actually received.
	int GetLastError(void);                // GetLastError returns the WSA error value for the last operation
   //char* GetLastErrorStr(void);
   BOOL Bind(u_short port);               // Bind binds the socket to the specified port. Returns TRUE on success.
   Transport& Accept(void);
   int IsInvalid(void);
   char *GetPeername(int flg=0);
   int type;

protected:
   SOCKET s;
   sockaddr_in peer,local;

private:
   int lasterr;
   static int socket_count;
};

class ConstantRate : public Thread, public Transport
{
public:
   ConstantRate(UINT rate=0, UINT blocksize=0); // Constructor initializes the object. rate is the datarate in bits per second and blocksize the amount of data sent in one packet in Byte.
   ~ConstantRate();                             // Destructor deletes the object.
   void SetTransferBuffer(class Buffer *buf);   // SetTransferbuffer and GetTransferBuffer set and retrieve the Buffer used by the constantrate class.
   class Buffer *GetTransferBuffer();      
   BOOL ChangeRate(UINT newrate);               // ChangeRate and ChangeBlockSz set new parameters for the sender. Both return TRUE on success. newrate in bits/s, newsz in Bytes.
   BOOL ChangeBlockSz(UINT newsz); 
   BOOL StartCRSend(void);                      // StartCRSend and StopCRSend start and stop the Constantrate sender. Both return TRUE on success.
   BOOL StopCRSend(void);
   BOOL StartReceive(void);                     // StartReceive and StopReceive start and stop the receiver.
   BOOL StopReceive(void);
   UINT GetRate(void);                          // GetRate and GetBlockSize retrieve the current sender parameters. Rate in bps, size in KB
   UINT GetBlockSize(void);
   void Run();                                  // Internal use only. Implements the receiver.
   virtual void OnOverflow(void) { }
   virtual void OnDisconnect(void) { }
   virtual void OnReceive(char *adr, int size) { }

protected:
   char *peername;
   Transport client;
   
private:
   Buffer *buffer;
   UINT r,blocksz;
	UINT delay;
   TIMECAPS tc;
   MMRESULT timer;
   BOOL sending,thrstarted,timestarted;
};

#endif
